/*
 * fmc228_regs.h
 *
 */

#ifndef __FMC228_H_
#define __FMC228_H_

#include "devices.h"

#define AMC_COMMON_BASE			0x0000
#define LMK04828_BASE			0x1000
#define HMC835_BASE			0x2000
#define AD9234_0_BASE			0x3000
#define AD9234_1_BASE			0x4000
#define GPIO_BASE			0x5000
#define JESD204_0_BASE			0x6000
#define JESD204_1_BASE			0x7000
#define ADC0_RD_BASE			0x8000
#define ADC1_RD_BASE			0x9000
	
#define GPIO_ORDERING_OPTION_B        	0x00000
#define GPIO_VCXO_EN	        	0x00004
#define GPIO_TRIG_CTRL			0x00010
#define GPIO_USER_LED			0x00014
#define GPIO_ADC_PDWN	        	0x00020
#define GPIO_ADC_OVR	        	0x00024
#define GPIO_ADC_RANGE_CLR		0x00028
#define GPIO_ADC_0_A_RANGE       	0x0002C
#define GPIO_ADC_0_B_RANGE       	0x00030
#define GPIO_ADC_1_A_RANGE       	0x00034
#define GPIO_ADC_1_B_RANGE       	0x00038
#define GPIO_LMK04828_RST      		0x00040
#define GPIO_LMK04828_STATUS    	0x00044
#define GPIO_LMK04828_SYNC_CTRL		0x0004C 
#define GPIO_HMC835_LD_STATUS   	0x00050
#define GPIO_HMC835_CTRL      		0x00054
#define GPIO_JESD204_RESET     		0x00070
#define GPIO_ADC_ACQ_CTRL       	0x00074
#define GPIO_ACQ_FIFO_STAT      	0x00078
#define GPIO_SCRATCH    		0x00FF4
#define GPIO_VER      			0x00FF8
#define GPIO_SIG		      	0x00FFC

/* BIT MASKS */
#define ORDERING_OPTION_B_MASK		0x00000001

/*
 * SHORT VERSION OF COMMON REGISTERS
 */
#define REG_SIG				GPIO_BASE + GPIO_SIG
#define REG_VER				GPIO_BASE + GPIO_VER
#define REG_SCRATCH			GPIO_BASE + GPIO_SCRATCH


const reg_with_name_t gpio_regs[] =
{
	{ 0x00000, 1, 0, 0, "[0] ORDERING_OPTION_B" },
	{ 0x00004, 3, 0, 0, "[0] VCXO_EN" },
	{ 0x00010, 3, 0, 0, "TRIG_CTRL" },
	{ 0x00014, 3, 0, 0, "[2:0] USER_LED" },
	{ 0x00020, 3, 0, 0, "[1] ADC1_PDWN [0] ADC0_PDWN" },
	{ 0x00024, 1, 0, 0, "ADC_OVR" },
	{ 0x00028, 3, 0, 0, "[1] ADC data format 1:binary offset; 0:2's complemtn [0] ADC_RANGE_CLR, self-clear" },
	{ 0x0002C, 1, 0, 0, "ADC_0_A_RANGE" },
	{ 0x00030, 1, 0, 0, "ADC_0_B_RANGE" },
	{ 0x00034, 1, 0, 0, "ADC_1_A_RANGE" },
	{ 0x00038, 1, 0, 0, "ADC_1_B_RANGE" },
	{ 0x00040, 3, 0, 0, "[0] LMK04828_RST" },
	{ 0x00044, 1, 0, 0, "[1] LMK04828_LD2 [0] LMK04828_LD1" },
	{ 0x0004C, 3, 0, 0, "[0] LMK04828_SYNC_CTRL" },
	{ 0x00050, 1, 0, 0, "[0] HMC835_LD" },
	{ 0x00054, 3, 1, 1, "[0] HMC835_CE" },
	{ 0x00070, 3, 0, 7, "JESD204_RESET" },
	{ 0x00074, 3, 0, 0, "[3] ADC_FIFO_WR_EN [2] ADC_FIFO_RST" },
	{ 0x00078, 3, 0, 0, "ADC0_FIFO: [14] empty [13] almost_empty [12] full [11:0] data_count; [30:16] ADC1" },
	{ 0x00FF4, 3, 0, 0, "SCRATCH" },
	{ 0x00FF8, 1, 0, 0, "VER" },
	{ 0x00FFC, 1, 0, 0, "SIG" },
	{ 0, 0, 0, 0, NULL}
};

#endif 
/* __FMC228_H_ */
