#ifndef __JESD204_H_
#define __JESD204_H_

#define JESD204_VERSION			0x00000
#define JESD204_RESET			0x00004
#define JESD204_ILA_SUPPORT		0x00008
#define JESD204_SCRAMBLING		0x0000C
#define JESD204_SYSREF_HANDLING		0x00010
#define JESD204_ILA_MULTIFRAMES		0x00014
#define JESD204_TEST_MODES		0x00018
#define JESD204_LINK_ERROR_STATUS	0x0001C
#define JESD204_OCTETS_PER_FRAME	0x00020
#define JESD204_FRAMES_PER_MULTIFRAMES	0x00024
#define JESD204_LANES_IN_USE		0x00028
#define JESD204_SUBCLASS_MODE		0x0002C
#define JESD204_RX_BUFFER_DELAY		0x00030
#define JESD204_ERROR_REPORTING		0x00034
#define JESD204_SYNC_STATUS		0x00038
#define JESD204_LANE0_ILA_CONFIG_DATA0	0x00800
#define JESD204_LANE0_ILA_CONFIG_DATA1	0x00804
#define JESD204_LANE0_ILA_CONFIG_DATA2	0x00808
#define JESD204_LANE0_ILA_CONFIG_DATA3	0x0080C
#define JESD204_LANE0_ILA_CONFIG_DATA4	0x00810
#define JESD204_LANE0_ILA_CONFIG_DATA5	0x00814
#define JESD204_LANE0_ILA_CONFIG_DATA6	0x00818
#define JESD204_LANE0_ILA_CONFIG_DATA7	0x0081C
#define JESD204_LANE0_TEST_MODE_ERR_CNT	0x00820
#define JESD204_LANE0_LINK_ERROR_CNT	0x00824
#define JESD204_LANE0_TEST_MODE_ILA_CNT 0x00828
#define JESD204_LANE0_TEST_MODE_MF_CNT	0x0082C
#define JESD204_LANE0_BUFFER_ADJUST	0x00830

/* rx only */
const reg_with_name_t xilinx_jesd204_regs[] =
{
	{ 0x00000, 1, 0, 0, "JESD204_VERSION" },
	{ 0x00004, 3, 1, 0, "JESD204_RESET" },	// reset the core
	{ 0x00008, 3, 1, 1, "JESD204_ILA_SUPPORT" },
	{ 0x0000C, 3, 1, 0, "JESD204_SCRAMBLING" },
	{ 0x00010, 3, 0x10000, 0, "JESD204_SYSREF_HANDLING" }, // change
	{ 0x00014, 0, 0, 0x03, "JESD204_ILA_MULTIFRAMES" },
	{ 0x00018, 3, 0, 0, "JESD204_TEST_MODES" },
	{ 0x0001C, 1, 0, 0, "JESD204_LINK_ERROR_STATUS(Lanes 0 to 7)" },
	{ 0x00020, 3, 0, 0, "JESD204_OCTETS_PER_FRAME" }, // change
	{ 0x00024, 3, 0x1F, 0, "JESD204_FRAMES_PER_MULTIFRAMES" }, // change
	{ 0x00028, 3, 0xF, 3, "JESD204_LANES_IN_USE" }, // change
	{ 0x0002C, 3, 1, 1, "JESD204_SUBCLASS_MODE" },
	{ 0x00030, 3, 0, 0, "JESD204_RX_BUFFER_DELAY" },
	{ 0x00034, 3, 0x101, 0, "JESD204_ERROR_REPORTING" }, // change
	{ 0x00038, 1, 0, 0, "JESD204_SYNC_STATUS" },
	{ 0x0003C, 0, 0, 0, "JESD204_LINK_ERROR_STATUS(Lanes 8 to 11)" },
	{ 0x00800, 1, 0, 0, "JESD204_LANE0_ILA_CONFIG_DATA0" },
	{ 0x00804, 1, 0, 0, "JESD204_LANE0_ILA_CONFIG_DATA1" },
	{ 0x00808, 1, 0, 0, "JESD204_LANE0_ILA_CONFIG_DATA2" },
	{ 0x0080C, 1, 0, 0, "JESD204_LANE0_ILA_CONFIG_DATA3" },
	{ 0x00810, 1, 0, 0, "JESD204_LANE0_ILA_CONFIG_DATA4" },
	{ 0x00814, 1, 0, 0, "JESD204_LANE0_ILA_CONFIG_DATA5" },
	{ 0x00818, 1, 0, 0, "JESD204_LANE0_ILA_CONFIG_DATA6" },
	{ 0x0081C, 1, 0, 0, "JESD204_LANE0_ILA_CONFIG_DATA7" },
	{ 0x00820, 1, 0, 0, "JESD204_LANE0_TEST_MODE_ERR_CNT" },
	{ 0x00824, 1, 0, 0, "JESD204_LANE0_LINK_ERROR_CNT" },
	{ 0x00828, 1, 0, 0, "JESD204_LANE0_TEST_MODE_ILA_CNT" },
	{ 0x0082C, 1, 0, 0, "JESD204_LANE0_TEST_MODE_MF_CNT" },
	{ 0x00830, 1, 0, 0, "JESD204_LANE0_BUFFER_ADJUST" },
	{ 0x00840, 1, 0, 0, "JESD204_LANE1_ILA_CONFIG_DATA0" },
	{ 0x00844, 1, 0, 0, "JESD204_LANE1_ILA_CONFIG_DATA1" },
	{ 0x00848, 1, 0, 0, "JESD204_LANE1_ILA_CONFIG_DATA2" },
	{ 0x0084C, 1, 0, 0, "JESD204_LANE1_ILA_CONFIG_DATA3" },
	{ 0x00850, 1, 0, 0, "JESD204_LANE1_ILA_CONFIG_DATA4" },
	{ 0x00854, 1, 0, 0, "JESD204_LANE1_ILA_CONFIG_DATA5" },
	{ 0x00858, 1, 0, 0, "JESD204_LANE1_ILA_CONFIG_DATA6" },
	{ 0x0085C, 1, 0, 0, "JESD204_LANE1_ILA_CONFIG_DATA7" },
	{ 0x00860, 1, 0, 0, "JESD204_LANE1_TEST_MODE_ERR_CNT" },
	{ 0x00864, 1, 0, 0, "JESD204_LANE1_LINK_ERROR_CNT" },
	{ 0x00868, 1, 0, 0, "JESD204_LANE1_TEST_MODE_ILA_CNT" },
	{ 0x0086C, 1, 0, 0, "JESD204_LANE1_TEST_MODE_MF_CNT" },
	{ 0x00870, 1, 0, 0, "JESD204_LANE1_BUFFER_ADJUST" },
	{ 0x00880, 1, 0, 0, "JESD204_LANE2_ILA_CONFIG_DATA0" },
	{ 0x00884, 1, 0, 0, "JESD204_LANE2_ILA_CONFIG_DATA1" },
	{ 0x00888, 1, 0, 0, "JESD204_LANE2_ILA_CONFIG_DATA2" },
	{ 0x0088C, 1, 0, 0, "JESD204_LANE2_ILA_CONFIG_DATA3" },
	{ 0x00890, 1, 0, 0, "JESD204_LANE2_ILA_CONFIG_DATA4" },
	{ 0x00894, 1, 0, 0, "JESD204_LANE2_ILA_CONFIG_DATA5" },
	{ 0x00898, 1, 0, 0, "JESD204_LANE2_ILA_CONFIG_DATA6" },
	{ 0x0089C, 1, 0, 0, "JESD204_LANE2_ILA_CONFIG_DATA7" },
	{ 0x008A0, 1, 0, 0, "JESD204_LANE2_TEST_MODE_ERR_CNT" },
	{ 0x008A4, 1, 0, 0, "JESD204_LANE2_LINK_ERROR_CNT" },
	{ 0x008A8, 1, 0, 0, "JESD204_LANE2_TEST_MODE_ILA_CNT" },
	{ 0x008AC, 1, 0, 0, "JESD204_LANE2_TEST_MODE_MF_CNT" },
	{ 0x008B0, 1, 0, 0, "JESD204_LANE2_BUFFER_ADJUST" },
	{ 0x008C0, 1, 0, 0, "JESD204_LANE3_ILA_CONFIG_DATA0" },
	{ 0x008C4, 1, 0, 0, "JESD204_LANE3_ILA_CONFIG_DATA1" },
	{ 0x008C8, 1, 0, 0, "JESD204_LANE3_ILA_CONFIG_DATA2" },
	{ 0x008CC, 1, 0, 0, "JESD204_LANE3_ILA_CONFIG_DATA3" },
	{ 0x008D0, 1, 0, 0, "JESD204_LANE3_ILA_CONFIG_DATA4" },
	{ 0x008D4, 1, 0, 0, "JESD204_LANE3_ILA_CONFIG_DATA5" },
	{ 0x008D8, 1, 0, 0, "JESD204_LANE3_ILA_CONFIG_DATA6" },
	{ 0x008DC, 1, 0, 0, "JESD204_LANE3_ILA_CONFIG_DATA7" },
	{ 0x008E0, 1, 0, 0, "JESD204_LANE3_TEST_MODE_ERR_CNT" },
	{ 0x008E4, 1, 0, 0, "JESD204_LANE3_LINK_ERROR_CNT" },
	{ 0x008E8, 1, 0, 0, "JESD204_LANE3_TEST_MODE_ILA_CNT" },
	{ 0x008EC, 1, 0, 0, "JESD204_LANE3_TEST_MODE_MF_CNT" },
	{ 0x008F0, 1, 0, 0, "JESD204_LANE3_BUFFER_ADJUST" },
	{ 0, 0, 0, 0, NULL}
};

#endif 
/* __JESD204_H_ */